﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls.UI;

namespace _1110783_RadVirtualGridHierarchy
{
    public partial class Form1 : Form
    {
        private IList<Employee> data = new List<Employee>();
        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.employeesTableAdapter.Fill(this.nwindDataSet.Employees);

            this.radVirtualGrid1.CellValueNeeded += radVirtualGrid1_CellValueNeeded;
            this.radVirtualGrid1.QueryHasChildRows += radVirtualGrid1_QueryHasChildRows;
            this.radVirtualGrid1.RowExpanding += radVirtualGrid1_RowExpanding;
            this.radVirtualGrid1.CellFormatting += radVirtualGrid1_CellFormatting;
            this.radVirtualGrid1.RowExpanded += RadVirtualGrid1_RowExpanded;
            LoadData();
            this.radVirtualGrid1.TableElement.RowHeight = 120;
            this.radVirtualGrid1.AutoSizeColumnsMode = VirtualGridAutoSizeColumnsMode.Fill;
        }

        private void RadVirtualGrid1_RowExpanded(object sender, VirtualGridRowExpandedEventArgs e)
        {
            e.ChildViewInfo.AutoSizeColumnsMode = VirtualGridAutoSizeColumnsMode.Fill;
        }

        private void LoadData()
        {
            Random random = new Random();
            for (int i = 0; i < this.nwindDataSet.Employees.Count; i++)
            {
                NwindDataSet.EmployeesRow row = this.nwindDataSet.Employees[i];
                Employee employee = new Employee();
                employee.Name = row.FirstName + " " + row.LastName;
                employee.Photo = GetImageFromBytes(row.Photo);
                employee.Salary = random.Next(45000);
                employee.HireDate = row.HireDate;
                employee.Title = row.Title;
                int rowCount = random.Next(20) + 1;

                for (int j = 0; j < rowCount; j++)
                {
                    employee.Sales.Add(new Sale()
                    {
                        Name = employee.Name,
                        ProductNumber = random.Next(1000),
                        Quantity = random.Next(50),
                        Discount = random.Next(100),
                        Total = random.Next(10000)
                    });
                }

                rowCount = random.Next(20) + 1;
                for (int k = 0; k < rowCount; k++)
                {
                    employee.Orders.Add(new Order()
                    {
                        ClientName = "Client " + random.Next(1000),
                        ClientNumber = random.Next(1000),
                        Address = "Address" + random.Next(50),
                    });
                }

                data.Add(employee);
            }
            this.radVirtualGrid1.RowCount = data.Count;
            this.radVirtualGrid1.ColumnCount = Employee.FieldNames.Length;
        }

        private void radVirtualGrid1_RowExpanding(object sender, VirtualGridRowExpandingEventArgs e)
        {
            if (e.RowIndex % 2 == 0)
            {
                e.ChildViewInfo.ColumnCount = Sale.FieldNames.Length;
                e.ChildViewInfo.RowCount = data[e.ChildViewInfo.ParentRowIndex].Sales.Count;
            }
            else
            {
                e.ChildViewInfo.ColumnCount = Order.FieldNames.Length;
                e.ChildViewInfo.RowCount = data[e.ChildViewInfo.ParentRowIndex].Orders.Count;
            }
        }

        private void radVirtualGrid1_CellFormatting(object sender, VirtualGridCellElementEventArgs e)
        {
            if (e.CellElement.ColumnIndex < 0)
            {
                return;
            }

            if (e.CellElement.Value is Image)
            {
                e.CellElement.Image = (Image)e.CellElement.Value;
                e.CellElement.ImageLayout = ImageLayout.Zoom;
                e.CellElement.Text = "";
            }
            else
            {
                e.CellElement.ResetValue(LightVisualElement.ImageProperty, Telerik.WinControls.ValueResetFlags.Local);
            }
        }

        private void radVirtualGrid1_QueryHasChildRows(object sender, VirtualGridQueryHasChildRowsEventArgs e)
        {
            e.HasChildRows = (e.ViewInfo == this.radVirtualGrid1.MasterViewInfo);
        }

        private void radVirtualGrid1_CellValueNeeded(object sender, VirtualGridCellValueNeededEventArgs e)
        {
            if (e.ViewInfo == this.radVirtualGrid1.MasterViewInfo)
            {
                if (e.ColumnIndex < 0)
                {
                    return;
                }

                e.FieldName = Employee.FieldNames[e.ColumnIndex];

                if (e.RowIndex == RadVirtualGrid.HeaderRowIndex)
                {
                    e.Value = e.FieldName;
                }
                else if (e.RowIndex >= 0)
                {
                    e.Value = data[e.RowIndex][e.ColumnIndex];
                    if (e.ColumnIndex == 2)
                    {
                        e.FormatString = "${0:#,###}";
                    }
                    else if (e.ColumnIndex == 3)
                    {
                        e.FormatString = "{0:MM/dd/yy}";
                    }
                }
            }
            else
            {
                if (e.ColumnIndex < 0)
                {
                    return;
                }

                if (e.ViewInfo.ParentRowIndex % 2 == 0)
                {
                    e.FieldName = Sale.FieldNames[e.ColumnIndex];

                    if (e.RowIndex == RadVirtualGrid.HeaderRowIndex)
                    {
                        e.Value = e.FieldName;
                    }
                    else if (e.RowIndex >= 0)
                    {
                        e.Value = data[e.ViewInfo.ParentRowIndex].Sales[e.RowIndex][e.ColumnIndex];
                        if (e.ColumnIndex == 1)
                        {
                            e.FormatString = "#{0}";
                        }
                        else if (e.ColumnIndex == 3)
                        {
                            e.FormatString = "{0:F2}%";
                        }
                        else if (e.ColumnIndex == 4)
                        {
                            e.FormatString = "${0}";
                        }
                    }
                }
                else
                {
                    e.FieldName = Order.FieldNames[e.ColumnIndex];

                    if (e.RowIndex == RadVirtualGrid.HeaderRowIndex)
                    {
                        e.Value = e.FieldName;
                    }
                    else if (e.RowIndex >= 0)
                    {
                        e.Value = data[e.ViewInfo.ParentRowIndex].Orders[e.RowIndex][e.ColumnIndex];
                        if (e.ColumnIndex == 1)
                        {
                            e.FormatString = "#{0}";
                        }
                    }
                }
            }
        }

        private Image GetImageFromBytes(byte[] bytes)
        {
            Image result = null;
            MemoryStream stream = null;

            try
            {
                stream = new MemoryStream(bytes, 78, bytes.Length - 78);
                result = Image.FromStream(stream);
            }
            catch
            {
                try
                {
                    stream = new MemoryStream(bytes, 0, bytes.Length);
                    result = Image.FromStream(stream);
                }
                catch
                {
                    result = null;
                }
            }
            finally
            {
                if (stream != null)
                    stream.Close();
            }

            return result;
        }
    }

    public class Employee
    {
        public static readonly string[] FieldNames = { "Photo", "Name", "Salary", "HireDate", "Title" };
        public Image Photo { get; set; }
        public string Name { get; set; }
        public decimal Salary { get; set; }
        public DateTime HireDate { get; set; }
        public string Title { get; set; }
        public List<Sale> Sales { get; private set; }

        public List<Order> Orders { get; private set; }

        public object this[int index]
        {
            get
            {
                switch (index)
                {
                    case 0:
                        return Photo;
                    case 1:
                        return Name;
                    case 2:
                        return Salary;
                    case 3:
                        return HireDate;
                    case 4:
                        return Title;
                    default:
                        return null;
                }
            }
        }

        public Employee()
        {
            this.Sales = new List<Sale>();
            this.Orders = new List<Order>();
        }
    }

    public class Sale
    {
        public static readonly string[] FieldNames = { "Name", "ProductNumber", "Quantity", "Discount", "Total" };
        public string Name { get; set; }
        public int ProductNumber { get; set; }
        public int Quantity { get; set; }
        public int Discount { get; set; }
        public int Total { get; set; }

        public object this[int index]
        {
            get
            {
                switch (index)
                {
                    case 0:
                        return Name;
                    case 1:
                        return ProductNumber;
                    case 2:
                        return Quantity;
                    case 3:
                        return Discount;
                    case 4:
                        return Total;
                    default:
                        return null;
                }
            }
        }
    }

    public class Order
    {
        public static readonly string[] FieldNames = { "ClientName", "ClientNumber", "Address"};
        public string ClientName { get; set; }
        public int ClientNumber { get; set; }
        public string Address { get; set; }

        public object this[int index]
        {
            get
            {
                switch (index)
                {
                    case 0:
                        return ClientName;
                    case 1:
                        return ClientNumber;
                    case 2:
                        return Address;
                    default:
                        return null;
                }
            }
        }
    }
}
